from json import loads
from unittest import TestLoader
import yaml
from testtools import TestCase
from testtools.matchers import (
    ContainsDict,
    Equals,
    MatchesDict,
)
from timeout import timeout
from utils import (
    run_maas_cli,
    wait_machines,
)

from maasserver.enum import NODE_STATUS

tests_order = [
    'test_add_rsd_pod',
    'test_composed_machines_commissioned',
    'test_allocate_rsd_machines',
    'test_deploy_rsd_machines',
    'test_delete_rsd_pod',
    'test_composed_machines_removed',
]


def sorting_method(ignored, first_test, second_test):
    return tests_order.index(first_test) - tests_order.index(second_test)


TestLoader.sortTestMethodsUsing = sorting_method

RSD_CONFIG = yaml.load(open('rsd.yaml'))
RSD_POD_TYPE = 'rsd'


class TestRSD(TestCase):

    def _wait_machines(self, status=None, num_expected=None):
        if num_expected is None:
            num_expected = RSD_CONFIG['composed_machines']
        return wait_machines(self, num_expected, status)

    def _run_maas_cli(self, args):
        return run_maas_cli(self, "rsd", args)

    def test_add_rsd_pod(self):
        output, _ = self._run_maas_cli([
            "pods",
            "create",
            "type={}".format(RSD_POD_TYPE),
            "power_address={}:{}".format(RSD_CONFIG['ip'],
                                         RSD_CONFIG['port']),
            "power_user={}".format(RSD_CONFIG['user']),
            "power_pass={}".format(RSD_CONFIG['pass']),
        ])
        rsd_pod = loads(output)
        self.assertThat(rsd_pod, ContainsDict(
            {
                "architectures": Equals(8 * ['amd64/generic']),
                "type": Equals(RSD_POD_TYPE),
                "total": MatchesDict(
                    {
                        "cores": Equals(352),
                        "local_storage": Equals(959999999999),
                        "memory": Equals(249984),
                        "local_disks": Equals(7),
                    }
                ),
                "available": MatchesDict(
                    {
                        "cores": Equals(176),
                        "local_storage": Equals(480000000003),
                        "memory": Equals(124992),
                        "local_disks": Equals(3),
                    }
                ),
                "used": MatchesDict(
                    {
                        "cores": Equals(176),
                        "local_storage": Equals(479999999996),
                        "memory": Equals(124992),
                        "local_disks": Equals(4),
                    }
                ),
                "capabilities": Equals(["fixed_local_storage"]),
            }
        ))

    @timeout(20*60)
    def test_composed_machines_commissioned(self):
        rsd_machines = self._wait_machines(NODE_STATUS.READY)
        hw = [
            {
                'memory': m['memory'], 'cpu_count': m['cpu_count']
            } for m in rsd_machines
        ]
        expected = [
            {'memory': 32768, 'cpu_count': 56},
            {'memory': 32768, 'cpu_count': 40},
            {'memory': 32768, 'cpu_count': 40},
            {'memory': 32768, 'cpu_count': 40}
        ]
        self.assertCountEqual(expected, hw)

    @timeout(20*60)
    def test_allocate_rsd_machines(self):
        rsd_machines = self._wait_machines(NODE_STATUS.READY)
        for machine in rsd_machines:
            self._run_maas_cli([
                "machines", "allocate", "system_id={}".format(
                    machine['system_id']
                )
            ])
        self._wait_machines(NODE_STATUS.ALLOCATED)

    @timeout(20*60)
    def test_deploy_rsd_machines(self):
        allocated_machines = self._wait_machines(NODE_STATUS.ALLOCATED)
        for machine in allocated_machines:
            self._run_maas_cli(
                ["machine", "deploy", machine['system_id']]
            )
        self._wait_machines(NODE_STATUS.DEPLOYED)

    def test_delete_rsd_pod(self):
        output, _ = self._run_maas_cli(["pods", "read"])
        pods = loads(output)
        rsd_pods = [p for p in pods if p['type'] == RSD_POD_TYPE]
        self.assertThat(len(rsd_pods), Equals(1))
        rsd_pod = rsd_pods[0]
        output, err = self._run_maas_cli(
            ["pod", "delete", "%s" % rsd_pod['id']]
        )
        self.assertThat(err, Equals(""))
        output, _ = self._run_maas_cli(["pods", "read"])
        # Verify that the pod was deleted
        pods = loads(output)
        self.assertThat(len(pods), Equals(0))

    def test_composed_machines_removed(self):
        output, _ = self._run_maas_cli(["machines", "read"])
        machines = loads(output)
        self.assertThat(len(machines), Equals(0))
